
// ********************************************************************
// Revision with Basys3 board
// ********************************************************************
//  * nexsys4 ddr and basys3 use different adc chennels 
//  * basys3: 6=>adc(0), 7=>adc(2), 14=>adc(1), 15=>adc(3)
//  * need to re-conifgure/re-generate a new XADC instance
// ********************************************************************

module chu_xadc_basys3_core
   (
    input  logic clk,
    input  logic reset,
    // slot interface
    input  logic cs,
    input  logic read,
    input  logic write,
    input  logic [4:0] addr,
    input  logic [31:0] wr_data,
    output logic [31:0] rd_data,
    // external signals 
    input  logic [3:0] adc_p,
    input  logic [3:0] adc_n
   );

   // signal declaration
   logic [4:0] channel;
   logic [6:0] daddr_in;
   logic eoc;
   logic rdy;
   logic [15:0]  adc_data; 
   logic [15:0] adc0_out_reg, adc1_out_reg, adc2_out_reg, adc3_out_reg;
   logic [15:0] tmp_out_reg , vcc_out_reg ;
   logic [31:0] r_data;
   
   // instantiate xadc
   xadc_fpro_basys3 xadc_unit (
      .dclk_in(clk),         // input logic dclk_in
      .reset_in(reset),      // input logic reset_in
      .di_in(16'h0000),      // input logic [15 : 0] di_in
      .daddr_in(daddr_in),   // input logic [6 : 0] daddr_in
      .den_in(eoc),          // input logic den_in
      .dwe_in(1'b0),         // input logic dwe_in
      .drdy_out(rdy),        // output logic drdy_out
      .do_out(adc_data),     // output logic [15 : 0] do_out
      .vp_in(1'b0),          // input logic vp_in
      .vn_in(1'b0),          // input logic vn_in
      .vauxp6(adc_p[0]),
      .vauxn6(adc_n[0]),
      .vauxp7(adc_p[2]),
      .vauxn7(adc_n[2]),
      .vauxp14(adc_p[1]),
      .vauxn14(adc_n[1]),
      .vauxp15(adc_p[3]),
      .vauxn15(adc_n[3]),
      .channel_out(channel), // output logic [4 : 0] channel_out
      .eoc_out(eoc),         // output logic eoc_out
      .alarm_out(),          // output logic alarm_out
      .eos_out(),            // output logic eos_out
      .busy_out()            // output logic busy_out
   );

   assign daddr_in = {2'b00, channel};
   
   // registers and decoding
   always_ff @(posedge clk, posedge reset)
      if (reset) begin
         adc0_out_reg <= 16'h0000;
         adc1_out_reg <= 16'h0000;
         adc2_out_reg <= 16'h0000;
         adc3_out_reg <= 16'h0000;
         tmp_out_reg <= 16'h0000;
         vcc_out_reg <= 16'h0000;
      end 
      else begin
         if (rdy && channel == 5'b10110)
            adc0_out_reg <= adc_data;
         if (rdy && channel == 5'b11110)
            adc1_out_reg <= adc_data;
         if (rdy && channel == 5'b10111)
            adc2_out_reg <= adc_data;
         if (rdy && channel == 5'b11111)
            adc3_out_reg <= adc_data;
         if (rdy && channel == 5'b00000)
            tmp_out_reg <= adc_data;
         if (rdy && channel == 5'b00001)
            vcc_out_reg <= adc_data;
     end
    
   // read multiplexing 
   always_comb
      case(addr[2:0])
         3'b000:
            r_data <= {16'h0000, adc0_out_reg};
         3'b001:
            r_data <= {16'h0000, adc1_out_reg};
         3'b010:
            r_data <= {16'h0000, adc2_out_reg};
         3'b011:
            r_data <= {16'h0000, adc3_out_reg};
         3'b100:
            r_data <= {16'h0000, tmp_out_reg};
         default:
            r_data <= {16'h0000, vcc_out_reg};
      endcase
      assign rd_data = r_data;
endmodule     


